// SPDX-License-Identifier: MIT
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

static inline double curtime(void) {
  struct timespec t;
  clock_gettime(CLOCK_MONOTONIC, &t);
  return t.tv_sec + t.tv_nsec * 1e-9;
}

double compute(size_t dim, long a[dim][dim], long b[dim][dim], long c[dim][dim],
               size_t iterations) {

  double time = 0;
  for (size_t n = 0; n < iterations; n++) {
    for (int j = 0; j < dim; j++) {
      for (int i = 0; i < dim; i++) {
        printf("\rProcessing element %d,%d...", i, j);
        double start = curtime();
        c[i][j] = a[i][j] * b[i][j];
        double end = curtime();
        time += end - start;
      }
    }
  }

  return time;
}

void init(size_t dim, long a[dim][dim], long b[dim][dim], long c[dim][dim]) {
  for (int j = 0; j < dim; j++)
    for (int i = 0; i < dim; i++)
      a[i][j] = b[i][j] = i * dim + j;
}

int main(int argc, char** argv) {
  size_t problem_size = 512;
  size_t iterations = 10;

  // Parse command line arguments. Refer to man page for details:
  // $ man 3 getopt
  int opt;
  while ((opt = getopt(argc, argv, "p:i:")) != -1) {
    switch (opt) {
    case 'p': problem_size = strtoul(optarg, NULL, 0); break;
    case 'i': iterations = strtoul(optarg, NULL, 0); break;
    default:
      fprintf(stderr, "usage: %s [-p problem_size] [-i iterations]\n", argv[0]);
      exit(EXIT_FAILURE);
    }
  }

  void* a = malloc(problem_size * problem_size * sizeof(long));
  void* b = malloc(problem_size * problem_size * sizeof(long));
  void* c = calloc(problem_size * problem_size, sizeof(long));

  init(problem_size, a, b, c);

  double time = compute(problem_size, a, b, c, iterations);
  printf("Took %f seconds\n", time);

  return 0;
}
